The rbindlistWithRownames() function uses the data.table::rbindlist()
function for a fast concatenation (row-binding) of a list of provided
data.frames and in addition also preserves their row names (if set).
The parameters are the same as for rbindlist().
Arguments
- l
A
listcontainingdata.frames that should be combined.- use.names
If
TRUEbinds by matching column names, ifFALSEby position."check"(default) warns if not all items have the same names in the same order. Seedata.table::rbindlist()for details.- fill
logical(1): ifTRUEfills missing columns with NAs orNULLfor missing list columns. By defaltFALSE.- idcol
Creates a column in the result showing which list item those rows cam from. See
data.table::rbindlist()for details.- ignore.attr
logical(1), defaultFALSE. WhenTRUE, allows binding columns with different attributes (e.g. class).
Note
Row names are dropped if duplicated row names are present, or if row
names are not defined for all data.frames in l.
The function uses .row_names_info() to guess wheter row names are really
set or just the default seq_len(nrow(x)) are used.